package oliot.aika.rajapinta;

/**
 * Sisäinen tilanvalvonta Aikaan.
 * @author  Vesa Lappalainen @version 1.0, 01.02.2003
 * @author Santtu Viitanen @version 1.1, 7.7.2011
 * @example
 * <pre name="test">
 *  Aika a1 = new Aika(12,15);
 *  a1.lisaa(55);  a1.toString() === "13:10";
 *  a1.aseta(12,15); a1.toString() === "12:15";
 * </pre>
 */
public class Aika implements AikaRajapinta, Cloneable {

    // private int h=0, m=0;
    private int yhtmin;

    /**
     * Alustaa ajan
     * @param h tunnit
     * @param m minuutit
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    public Aika(int h,int m) {
       aseta(h, m);
    }
    
    /**
     * Alustaa ajan muotoon 00:00
     * @example
     * <pre name="test">
     * new Aika().toString() === "00:00";
     * </pre>
     */
    public Aika() {
        yhtmin = 0;
    }

    /**
     * Asettaa uuden ajan ja pitää huolen että aika on aina oikeaa muotoa.
     * @param ih asetettava tuntimäärä
     * @param im asetettava minuuttimäärä
     * @return montako vuorokautta jäi yli
     * <pre name="test">
     *  Aika a1 = new Aika();
     *  a1.aseta(12,15); a1.toString() === "12:15"; 
     *  a1.aseta(15,45); a1.toString() === "15:45";
     *  a1.aseta(-49,-125); a1.toString() === "20:55";
     * </pre>
     */
    @Override
    public final int aseta(int ih, int im) {
        yhtmin = ih*60 + im;
        int vrk = 0;
        return vrk; // motako vuorokautta jäi yli
    }

    
    /**
     * Lisää aikaan valitun minuuttimäärän
     * @param lisaMin lisättävä minuuttimäärä
     * @example
     * <pre name="test">
     *  Aika a1 = new Aika(13,16);
     *  a1.lisaa(27);    a1.toString() === "13:43";
     *  a1.lisaa(39);    a1.toString() === "14:22";
     * </pre>
     */
    @Override
    public void lisaa(int lisaMin)  {
        this.yhtmin += lisaMin;
    }
    
    
    /**
     * @return tuntiosuus
     */
    @Override
    public int getH() {
        return (yhtmin / 60) % 24;
    }
    
    
    /**
     * @return minosuus
     */
    @Override
    public int getM() {
        return yhtmin % 60;
    }
    
    
    /**
     * @return aika merkkijonona muodossa 12:05
     * @example
     * <pre name="test">
     * new Aika(12,15).toString() === "12:15";
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%02d:%02d",getH(), getM());
    }

    /**
     * 
     * @param obj aika johon verrataan
     * @return onko samat vai ei
     */
    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof AikaRajapinta)) return false;
        Aika a = (Aika)obj;
        // if (a.getM() != this.getM() ) return false;
        // if (a.getH() != this.getH() ) return false;
        return this.hashCode() == a.hashCode();
    }
    
    
    @Override
    public int hashCode() {
        return 3600*getH() + 60*getH();
    }

    
    @Override
    public Aika clone() {
        Aika a = null;
        try {
            a = (Aika)super.clone();
            // a.aseta(h,m); // tätä ei tarvitse, koska super laittaa int yms. attr.
        } catch (CloneNotSupportedException e) { // 
        }
        return a;
    }
    
    /**
     * @param thus aika johon lisätään
     * @param m paljonko lisätään
     */
    public static void lisaa(Aika thus, int m) {
        thus.yhtmin += m;
    }
    
    
    /**
     * Tutkitaan aika-luokkaa
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        AikaRajapinta a0 = new Aika();
        System.out.println(a0);
        a0.aseta(12,67);
        System.out.println(a0);
        AikaRajapinta a1 = new Aika(12,89);
        System.out.println(a1);
        Aika a3 = new Aika(12,15);
        Aika a4 = new Aika(12,15);
        
        if (a3.equals(a4)) System.out.println("Samoja on");
        
        Aika a5 = a3.clone();
        System.out.println(a5);
        
        a3.lisaa(5);
        System.out.println(a3);
        lisaa(a3, 5);
        
    }
}
