package jonot;

/**
 * Esimerkki dynaamisesta taulukosta
 * @author vesal
 * @version 2.3.2021
 *
 */
public class Taulukko {
    
    /** Luokka täyden taulukon poikkeusta varten  */
    public static class TaulukkoTaysiException extends Exception {
        private static final long serialVersionUID = 1L;

        TaulukkoTaysiException(String viesti) {
            super(viesti);
        }
    }
    
    
    private int lkm; 
    private String[] alkiot; 
    
    /**
     * Luodaan taulukko jonka alkukokona on koko
     * @param koko jolla aloitetaan
     */
    public Taulukko(int koko) {
        alkiot = new String[koko];
        lkm = 0;
    }
    
    
    /**
     * Lisätään uusi alkio tietorakenteeseen
     * @param luku listtävä alkio
     * @throws TaulukkoTaysiException jos tila loppuu
     * @example
     * <pre name="test">
     *   #THROWS Taulukko.TaulukkoTaysiException
     *   Taulukko luvut = new Taulukko(2);
     *   luvut.toString() === "";
     *   luvut.lisaa(5);
     *   luvut.toString() === "5";
     *   luvut.lisaa(23);
     *   luvut.toString() === "5,23";
     *   luvut.lisaa(734); #THROWS Taulukko.TaulukkoTaysiException
     * </pre>
     */
    public void lisaa(String luku) throws TaulukkoTaysiException {
        if (lkm >= alkiot.length) 
            throw new TaulukkoTaysiException("Tila loppu, ei voinut lisätä: " + luku);
        alkiot[lkm] = luku;
        lkm++;
    }
    
    
    /**
     * @param i paikka josta annetaan
     * @return alkio paikasta i
     * @throws IndexOutOfBoundsException jos otetaan väärästä paikasta
     */
    public String get(int i) throws IndexOutOfBoundsException {
        if ((i < 0)  || (lkm <= i)) throw new IndexOutOfBoundsException("i = " + i);
        return alkiot[i];    
    }

    
    @Override
    public String toString() {
        StringBuilder s = new StringBuilder("");
        String erotin = "";
        for (int i = 0; i < lkm; i++) {
            s.append(erotin);
            s.append(alkiot[i]);
            erotin = ",";
        }
        return s.toString();
    }
    
    
    /**
     * Kokeillaan taulukkoa
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Taulukko luvut = new Taulukko(30);
        try {
            luvut.lisaa("kissa");
            luvut.lisaa("koira");
            luvut.lisaa("kana");
            String a = luvut.get(2);
            System.out.println(a); 
        } catch (TaulukkoTaysiException e) {
            System.err.println(e.getMessage());
        } catch (IndexOutOfBoundsException e) {
            System.err.println(e.getMessage());
        }
        
        System.out.println(luvut);  // toString
    }
}
