package fxKerho;

import java.awt.Desktop;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.TextAreaOutputStream;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.text.Font;
import kerho.Harrastus;
import kerho.Jasen;
import kerho.Kerho;
import kerho.SailoException;

/**
 * Luokka kerhon käyttöliittymän tapahtumien hoitamiseksi.
 * 
 * @author vesal
 * @version 31.12.2015
 * @version 5.1.2015
 */
public class KerhoGUIController implements Initializable {

    @FXML private TextField hakuehto;
    @FXML private ComboBoxChooser<String> cbKentat;
    @FXML private Label labelVirhe;
    @FXML private ScrollPane panelJasen;
    @FXML private ListChooser<Jasen> chooserJasenet;
    
    private String kerhonnimi = "kelmit";
    
    
    @Override
    public void initialize(URL url, ResourceBundle bundle) {
        alusta();      
    }

    
    @FXML private void handleHakuehto() {
        String hakukentta = cbKentat.getSelectedText();
        String ehto = hakuehto.getText(); 
        if ( ehto.isEmpty() )
            naytaVirhe(null);
        else
            naytaVirhe("Ei osata vielä hakea " + hakukentta + ": " + ehto);
    }
    
    
    @FXML private void handleTallenna() {
        tallenna();
    }
    
    
    @FXML private void handleAvaa() {
        avaa();
    }
    
    
    @FXML private void handleTulosta() {
        TulostusController tulostusCtrl = TulostusController.tulosta(null); 
        tulostaValitut(tulostusCtrl.getTextArea()); 
    } 

    
    @FXML private void handleLopeta() {
        tallenna();
        Platform.exit();
    } 

    
    @FXML private void handleUusiJasen() {
        uusiJasen();
    }
    
    
    @FXML private void handleMuokkaaJasen() {
        ModalController.showModal(KerhoGUIController.class.getResource("JasenDialogView.fxml"), "Jäsen", null, "");
    }
    
    
    @FXML private void handlePoistaJasen() {
        Dialogs.showMessageDialog("Vielä ei osata poistaa jäsentä");
    }
    
     
    @FXML private void handleUusiHarrastus() {
        uusiHarrastus();  
    }
    

    @FXML private void handleMuokkaaHarrastus() {
        ModalController.showModal(KerhoGUIController.class.getResource("HarrastusDialogView.fxml"), "Harrastus", null, "");
    }
    

    @FXML private void handlePoistaHarrastus() {
        Dialogs.showMessageDialog("Ei osata vielä poistaa harrastusta");
    }
    

    @FXML private void handleApua() {
        avustus();
    }
    

    @FXML private void handleTietoja() {
        // Dialogs.showMessageDialog("Ei osata vielä tietoja");
        ModalController.showModal(KerhoGUIController.class.getResource("AboutView.fxml"), "Kerho", null, "");
    }
    

//===========================================================================================    
// Tästä eteenpäin ei käyttöliittymään suoraan liittyvää koodia    
    
    private Kerho kerho;
    private Jasen jasenKohdalla;
    private TextArea areaJasen = new TextArea();
    
    
    /**
     * Tekee tarvittavat muut alustukset, nyt vaihdetaan GridPanen tilalle
     * yksi iso tekstikenttä, johon voidaan tulostaa jäsenten tiedot.
     * Alustetaan myös jäsenlistan kuuntelija 
     */
    protected void alusta() {
        panelJasen.setContent(areaJasen);
        areaJasen.setFont(new Font("Courier New", 12));
        panelJasen.setFitToHeight(true);
        
        chooserJasenet.clear();
        chooserJasenet.addSelectionListener(e -> naytaJasen());
    }

    
    private void naytaVirhe(String virhe) {
        if ( virhe == null || virhe.isEmpty() ) {
            labelVirhe.setText("");
            labelVirhe.getStyleClass().removeAll("virhe");
            return;
        }
        labelVirhe.setText(virhe);
        labelVirhe.getStyleClass().add("virhe");
    }
    
    
    private void setTitle(String title) {
        ModalController.getStage(hakuehto).setTitle(title);
    }
    
    
    /**
     * Alustaa kerhon lukemalla sen valitun nimisestä tiedostosta
     * @param nimi tiedosto josta kerhon tiedot luetaan
     */
    protected void lueTiedosto(String nimi) {
        kerhonnimi = nimi;
        setTitle("Kerho - " + kerhonnimi);
        try {
            kerho.lueTiedostosta(nimi);
            hae(0);
        } catch (SailoException e) {
            Dialogs.showMessageDialog(e.getMessage());
        }
    }

    
    /**
     * Kysytään tiedoston nimi ja luetaan se
     * @return true jos onnistui, false jos ei
     */
    public boolean avaa() {
        String uusinimi = KerhonNimiController.kysyNimi(null, kerhonnimi);
        if (uusinimi == null) return false;
        lueTiedosto(uusinimi);
        return true;
    }

    
    /**
     * Tietojen tallennus
     */
    private void tallenna() {
        try {
            kerho.tallenna();
        } catch (SailoException e) {
            Dialogs.showMessageDialog(e.getMessage());
        }
    }


    /**
     * Tarkistetaan onko tallennus tehty
     * @return true jos saa sulkea sovelluksen, false jos ei
     */
    public boolean voikoSulkea() {
        tallenna();
        return true;
    }
    
    
    /**
     * Näyttää listasta valitun jäsenen tiedot, tilapäisesti yhteen isoon edit-kenttään
     */
    private void naytaJasen() {
        jasenKohdalla = chooserJasenet.getSelectedObject();

        if (jasenKohdalla == null) return;

        areaJasen.setText("");
        try (PrintStream os = TextAreaOutputStream.getTextPrintStream(areaJasen)) {
            tulosta(os, jasenKohdalla);
        }
    }



    /**
     * Hakee jäsenten tiedot listaan
     * @param jnro jäsenen numero, joka aktivoidaan haun jälkeen
     */
    protected void hae(int jnro) {
        chooserJasenet.clear();

        int index = 0;
        for (int i = 0; i < kerho.getJasenia(); i++) {
            Jasen jasen = kerho.annaJasen(i);
            if (jasen.getTunnusNro() == jnro) index = i;
            chooserJasenet.add(jasen.getNimi(), jasen);
        }
        chooserJasenet.setSelectedIndex(index); // tästä tulee muutosviesti joka näyttää jäsenen
    }


    /**
     * Luo uuden jäsenen jota aletaan editoimaan 
     */
    protected void uusiJasen() {
        Jasen uusi = new Jasen();
        uusi.rekisteroi();
        uusi.vastaaAkuAnkka();
        try {
            kerho.lisaa(uusi);
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia uuden luomisessa " + e.getMessage());
            return;
        }
        hae(uusi.getTunnusNro());
    }
    

    /** 
     * Tekee uuden tyhjän harrastuksen editointia varten 
     */ 
    public void uusiHarrastus() { 
        if ( jasenKohdalla == null ) return;  
        Harrastus har = new Harrastus();  
        har.rekisteroi();  
        har.vastaaPitsinNyplays(jasenKohdalla.getTunnusNro());  
        kerho.lisaa(har);  
        hae(jasenKohdalla.getTunnusNro());          
    } 
 
    
    
    /**
     * @param kerho Kerho jota käytetään tässä käyttöliittymässä
     */
    public void setKerho(Kerho kerho) {
        this.kerho = kerho;
        naytaJasen();
    }

    
    /**
     * Näytetään ohjelman suunnitelma erillisessä selaimessa.
     */
    private void avustus() {
        Desktop desktop = Desktop.getDesktop();
        try {
            URI uri = new URI("https://tim.jyu.fi/view/kurssit/tie/ohj2/2019k/ht/vesal");
            desktop.browse(uri);
        } catch (URISyntaxException e) {
            return;
        } catch (IOException e) {
            return;
        }
    }

    /**
     * Tulostaa jäsenen tiedot
     * @param os tietovirta johon tulostetaan
     * @param jasen tulostettava jäsen
     */
    public void tulosta(PrintStream os, final Jasen jasen) {
        os.println("----------------------------------------------");
        jasen.tulosta(os);
        List<Harrastus> harrastukset = kerho.annaHarrastukset(jasen);
        for (Harrastus har: harrastukset) 
            har.tulosta(os);
        os.println("----------------------------------------------");
    }
    
    
    /**
     * Tulostaa listassa olevat jäsenet tekstialueeseen
     * @param text alue johon tulostetaan
     */
    public void tulostaValitut(TextArea text) {
        try (PrintStream os = TextAreaOutputStream.getTextPrintStream(text)) {
            os.println("Tulostetaan kaikki jäsenet");
            for (int i = 0; i < kerho.getJasenia(); i++) {
                Jasen jasen = kerho.annaJasen(i);
                tulosta(os, jasen);
                os.println("\n\n");
            }
        }
    }
}
