package funktio;
import static java.lang.Math.*; 

/**
 * Ohjelmalla integroidaan numeerisesti 
 * @author Vesa Lappalainen
 * @version 1.0 
 */
@SuppressWarnings("javadoc")
public class Integroi2 {

    /**
     * Rajapinta kaikille funktiolle R->R
     */
    public interface FunktioRR {
        double f(double x);
    }

    public static class SinFun implements FunktioRR {
        @Override
        public double f(double x) {
            return sin(x);
        }
    }
    
    public static class P2 implements FunktioRR  {
        private double a,b,c;
        
        public P2(double a, double b, double c) {
            this.a = a;
            this.b = b;
            this.c = c;
        }
        @Override
        public double f(double x) {
            return a*x*x + b*x + c;
        }
    }
    
    /**
     * Integroidaan sin(x) välillä x1-x2
     * @param f jota integroidaan
     * @param x1 alkuarvo
     * @param x2 loppuarvo
     * @param tiheys monellako askeleella
     * @return likiarvo integraalille
     */
    public static double integroi(FunktioRR f, double x1, double x2, int tiheys) {
        double summa = 0;
        double dx = (x2-x1)/tiheys;
        for (double x = x1 + dx/2; x < x2; x += dx) {
            double h = f.f(x);
            double ala = h*dx;
            summa += ala;
        }
        return summa;
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        SinFun sinf = new SinFun();
        double ala = integroi(sinf, 0, PI, 10000);
        System.out.printf("%17.15f%n", ala);
        P2 p2 = new P2(3,-2,5);
        ala = integroi(p2, 0, 1, 10000);
        System.out.printf("%17.15f%n", ala);
        
        FunktioRR exp = new FunktioRR() {
            @Override
            public double f(double x) { return Math.exp(x); }
        };
        ala = integroi(exp, 0, 1, 10000);
        System.out.printf("%17.15f%n", ala);

        FunktioRR p3 = x -> x*x*x;
        ala = integroi(p3, 0, 1, 10000);
        System.out.printf("%17.15f%n", ala);

        ala = integroi(x -> x*x, 0, 1, 10000);
        System.out.printf("%17.15f%n", ala);


        
    }
    
}
