package fxKerho;

import java.net.URL;
import java.util.ResourceBundle;

import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.ModalControllerInterface;
import fi.jyu.mit.ohj2.Mjonot;
import javafx.scene.Node;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import javafx.stage.Stage;
import kerho.Jasen;

/**
 * Kysytään jäsenen tiedot luomalla sille uusi dialogi
 * 
 * @author vesal
 * @version 11.1.2016
 *
 */
public class JasenDialogController implements ModalControllerInterface<Jasen>,Initializable  {

    @FXML private TextField editNimi;
    @FXML private TextField editHetu;
    @FXML private TextField editKatuosoite;
    @FXML private TextField editPostinumero;   
    @FXML private GridPane gridJasen;
    @FXML private Label labelVirhe;

    @Override
    public void initialize(URL url, ResourceBundle bundle) {
        alusta();  
    }
    
    @FXML private void handleOK() {
        if ( jasenKohdalla != null && jasenKohdalla.getNimi().trim().equals("") ) {
            naytaVirhe("Nimi ei saa olla tyhjä");
            return;
        }
        ModalController.closeStage(labelVirhe);
    }

    
    @FXML private void handleCancel() {
        jasenKohdalla = null;
        ModalController.closeStage(labelVirhe);
    }

// ========================================================    
    private Jasen jasenKohdalla;
    private TextField edits[];
    private int kentta = 1;
   

    /**
     * Tyhjentään tekstikentät 
     * @param edits tauluko jossa tyhjennettäviä tektsikenttiä
     */
    public static void tyhjenna(TextField[] edits) {
        for (TextField edit : edits)
            edit.setText("");
    }

    
    private static Jasen apujasen = new Jasen();  // kuten vakio

    /**
     * Luodaan GridPaneen jäsenen tiedot
     * @param grid mihin tiedot luodaan
     * @return luodut tekstikentät
     */
    public static TextField[] luoKentat(GridPane grid) {
        grid.getChildren().clear();
        TextField[] fedits = new TextField[apujasen.getKenttia()];
        
        for (int i=0, k = apujasen.ekaKentta(); k < apujasen.getKenttia(); k++, i++) {
            Label label = new Label(apujasen.getKysymys(k));
            grid.add(label, 0, i);
            TextField edit = new TextField();
            fedits[k] = edit;
            edit.setId("e"+k);   // osataan liittää jäsenessä oikeaan paikkaan
            grid.add(edit, 1, i);
        }
        return fedits;
    }
    
    
    
    /**
     * Tekee tarvittavat muut alustukset. Mm laittaa edit-kentistä tulevan
     * tapahtuman menemään kasitteleMuutosJaseneen-metodiin ja vie sille
     * kentännumeron parametrina.
     */
    protected void alusta() {
        // edits = new TextField[]{editNimi, editHetu, editKatuosoite, editPostinumero};
        edits = luoKentat(gridJasen);
        
        for (TextField edit : edits) {
           if (edit != null)
                edit.setOnKeyReleased( e -> kasitteleMuutosJaseneen(edit));
        }
    }
    
    
    @Override
    public void setDefault(Jasen oletus) {
        jasenKohdalla = oletus;
        naytaJasen(edits, jasenKohdalla);
    }

    
    @Override
    public Jasen getResult() {
        return jasenKohdalla;
    }
    
    
    /**
     * Mitä tehdään kun dialogi on näytetty
     */
    @Override
    public void handleShown() {
        // editNimi.requestFocus();
        kentta = Math.max(apujasen.ekaKentta(), Math.min(kentta, apujasen.getKenttia()-1));
        edits[this.kentta].requestFocus();
    }
    
    
    private void naytaVirhe(String virhe) {
        if ( virhe == null || virhe.isEmpty() ) {
            labelVirhe.setText("");
            labelVirhe.getStyleClass().removeAll("virhe");
            return;
        }
        labelVirhe.setText(virhe);
        labelVirhe.getStyleClass().add("virhe");
    }

    /**
     * Palautetaan komponentin id:stä saatava luku
     * @param obj tutkittava komponentti
     * @param oletus mikä arvo jos id ei ole kunnollinen
     * @return komponentin id lukuna 
     */
    public static int getFieldId(Object obj, int oletus) {
        if ( !( obj instanceof Node)) return oletus;
        Node node = (Node)obj;
        return Mjonot.erotaInt(node.getId().substring(1),oletus);
    }
    
    /**
     * Käsitellään jäseneen tullut muutos
     * @param edit muuttunut kenttä
     */
    private void kasitteleMuutosJaseneen(TextField edit) {
        if (jasenKohdalla == null) return;
        int k = getFieldId(edit,apujasen.ekaKentta());
        String s = edit.getText();
        String virhe = null;
        virhe = jasenKohdalla.aseta(k, s);
        if (virhe == null) {
            Dialogs.setToolTipText(edit,"");
            edit.getStyleClass().removeAll("virhe");
            naytaVirhe(virhe);
        } else {
            Dialogs.setToolTipText(edit,virhe);
            edit.getStyleClass().add("virhe");
            naytaVirhe(virhe);
        }
    }
    
    
    /**
     * Näytetään jäsenen tiedot TextField komponentteihin
     * @param edits taulukko TextFieldeistä johon näytetään
     * @param jasen näytettävä jäsen
     */
    public static void naytaJasen(TextField[] edits, Jasen jasen) {
        if (jasen == null) return;
        for (int k = jasen.ekaKentta(); k < jasen.getKenttia(); k++) {
            edits[k].setText(jasen.anna(k));
        }
    }
   
    private void setKentta(int kentta) {
        this.kentta = kentta;
    }
    
    
    /**
     * Luodaan jäsenen kysymisdialogi ja palautetaan sama tietue muutettuna tai null
     * TODO: korjattava toimimaan
     * @param modalityStage mille ollaan modaalisia, null = sovellukselle
     * @param oletus mitä dataan näytetään oletuksena
     * @param kentta mikä kenttä saa fokuksen kun näytetään
     * @return null jos painetaan Cancel, muuten täytetty tietue
     */
    public static Jasen kysyJasen(Stage modalityStage, Jasen oletus, int kentta) {
        return ModalController.<Jasen, JasenDialogController>showModal(
                    JasenDialogController.class.getResource("JasenDialogView.fxml"),
                    "Kerho",
                    modalityStage, oletus, 
                    ctrl -> ctrl.setKentta(kentta)
                );
    }

}
