package testaaminen;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

/**
 * Esitellään 3 eri tapaa testata.
 *  - ComTest
 *  - JUnit
 *  - itse kirjoitettu testi
 * @author vesal
 * @version 20.1.2012
 */
public class Alkuluku {

	/**
	 * Aliohjelmalla tutkitaan onko parametrina tuotu
	 * luku alkuluku vai ei<br>
	 * Algoritmi: Jaetaan tutkittavaa lukua jakajilla 2,3,5,7...luku/2.
	 * Jos jokin jako menee tasan, niin ei alkuluku:
	 * 
	 * @param luku tutkittava luku
	 * @return luvun jolla jaollinen tai 1 jos alkuluku
	 * @example
	 * <pre name="test">
	 *   onkoAlkuluku(25)  === 5;
	 *   onkoAlkuluku(2)   === 1;
	 *   onkoAlkuluku(4)   === 2;
	 *   onkoAlkuluku(123) === 3;
	 *   onkoAlkuluku(7)   === 1;
	 * </pre>
	 */
	public static int onkoAlkuluku(int luku) {
		int jakaja = 2;
		int kasvatus = 1;
		if (luku == 2)
			return 1;                  // 0

		do {
			int jakojaannos = luku % jakaja;
			if (jakojaannos == 0)
				return jakaja;         // 1
			jakaja += kasvatus;        // 2
			kasvatus = 2;              // 3

		} while (jakaja < luku / 2);

		return 1;
	}

	
	/**
	 * Testin voi kirjoittaa myös itse suoraan käyttäen JUnittia
	 * Tämän esimerkin vika on että testikoodi jää käännettyyn koodiin.
	 * Yleensä testi kirjoitetaan eri tiedostoon.
	 */
	@Test
	public void testOnkoAlkuluku() {
		assertEquals(5, onkoAlkuluku(25));
		assertEquals(1, onkoAlkuluku(2));
		assertEquals(2, onkoAlkuluku(4));
		assertEquals(3, onkoAlkuluku(123));
		assertEquals(1, onkoAlkuluku(7));
	}

	
	/**
	 * Testialiohjelman testaamaan miten onkoAlkuluku toimii.
	 * Tulostaa virhen jos sellainen tulee.
	 * @param i  testattva luku
	 * @param tulos mikä pitäisi tulla tulokseksi
	 * @return 0 jos oikein, muuten 1
	 */
	private static int alkulukuTesti(int i, int tulos) {
		int t = onkoAlkuluku(i);
		if (t == tulos)
			return 0;
		System.out.println("Meni väärin " + i + ", tuli " + t + " piti tulla "
		        + tulos);
		return 1;
	}

	
	/**
	 * Testataan alkuluku-aliohjelmaa
	 * @param args ei käytössä
	 */
	public static void main(String[] args) {
		int v = 0;
		v += alkulukuTesti(1, 1);
		v += alkulukuTesti(2, 1);
		v += alkulukuTesti(3, 1);
		v += alkulukuTesti(4, 2);
		v += alkulukuTesti(5, 1);
		v += alkulukuTesti(6, 2);
		v += alkulukuTesti(7, 1);
		v += alkulukuTesti(23, 1);
		v += alkulukuTesti(25, 5);
		v += alkulukuTesti(123, 3);
		v += alkulukuTesti(1013, 1);

		if (v != 0)
			System.out.println("Vääriä " + v);
	}

}
