package parturi;

import java.io.PrintStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Random;



/**
 * Luokka, jolla tutkitaan saako jokaisell asiakkalle parturiajan
 * @author vesal
 * @version 16.1.2020
 *
 */
public class Parturi {
    
    private int[] sijoitetut;  // mitkä asiakkaat on sijoitettu
    private int[] asiakkaat;   // laskuri montako aikaa asikkaalla on
    private int[] kukaTulossa; // kuka tuossa mihinkäkin aikaan
    private int[] kukaTulossa8;// kuka tuossa mihinkäkin aikaan 8 tornin algortimilla
    private int[] kukaTulossaA;// kuka tuossa mihinkäkin aikaan alkusijoittelun torin algortimilla
    private int[] halukkaita;  // laskuri sille montako asiakasta vielä tulossa tähän aikaan
    private int[][] ajat;      // taulukko jossa 1 jokaista mahdollista aikaa varten
    private int[][] alkup;     // kopio taulukosta
    
    private int[][] neliot;
    private int sijoitettuAsiakkaita;
    private int sijoitettuAikoja;
    private int prosentti;      // arvonnan täyttöprosentti  
    private long rndseed = Long.MIN_VALUE;
    private static Random rnd = null;
    
    
    /**
     * Alustetaan ongelma n-kokoiseksi
     * @param n ongelman koko
     * @param pros millä todennäköisyysprosentilla tulee 1 ruutuun
     * @param seed satunnaislukugeneraatoorin siemen
     * @param salli0 sallitaanko 0 rivit/sarakkeet
     */
    public Parturi(int n, int pros, long seed, boolean salli0) {
        ajat = new int[n][n]; 
        arvo(pros, seed, salli0);
        alusta();
    }
    
    
    /**
     * @param ajat taulukko jolla ajat alustetaan
     */
    public Parturi(int ajat[][]) {
        this.ajat = ajat; 
        alusta();
    }
    
    
    /**
     * @return asiakkaiden lukumäärä
     */
    public int getNA() {
        return ajat.length;
    }
    
    /**
     * @return aikojen lukumäärä 
     */
    public int getNT() {
        return ajat[0].length;
    }
    
    
    /**
     * @return palautetaan käytetty siemenluku merkijonona tai jos ei ole alustettu arpomalla, niin tyhjä.
     */
    public String getSeed() {
        return rndseed != Long.MIN_VALUE ? ""+rndseed : "";
    }
    
    /**
     * Arpoo taulukkon 1:n pros todennäköisyydellä
     * @param pros millä prosentilla laitetaan 1 ruutuun
     * @param seed mitä käytetään satunnaisluvun lähteenä
     */
    private void arvo(int pros, long seed, boolean salli0) {
        this.prosentti = pros;
        if ( rnd == null )  rnd = new Random();
        rndseed = seed == 0 ? rnd.nextLong(): seed;
        rnd = new Random(rndseed); // en tiedä olisiko helpommin saanut seedin talteen
        for (var arivi : ajat) {
            int arvoja = 0;
            for (int t=0; t < arivi.length; t++) {
                if ( rnd.nextInt(100) < pros ) {
                    arivi[t] = 1;
                    arvoja++;
                }
            }        
            if ( arvoja == 0 && !salli0) arivi[rnd.nextInt(arivi.length)] = 1; 
        }
        if ( salli0 ) return;
        
        // tutkitaan 0 sarakkeet
        for (int t = 0; t < ajat[0].length; t++) {
            int arvoja = 0;
            for (int a=0; a < ajat.length; a++) {
                if ( ajat[a][t] > 0 ) arvoja++;
            }        
            if ( arvoja == 0 ) ajat[rnd.nextInt(ajat.length)][t] = 1; 
        }
    }

    
    
    private void luoAputaulut(int na, int nt) {
        sijoitetut = new int[na];
        asiakkaat = new int[na];
        kukaTulossa = new int[nt];
        halukkaita = new int[nt];
    }
    
    /**
     * Alustaa aputaulukot
     */
    private void alusta() {
        int na = ajat.length;
        int nt = ajat[0].length;
        alkup = new int[na][nt];
        luoAputaulut(na, nt);
        for (int a=0; a < ajat.length; a++) {
            var arivi = ajat[a];   
            for (int t=0; t < arivi.length; t++) {
                alkup[a][t] = arivi[t];
                if ( arivi[t] > 0 ) {
                    asiakkaat[a]++;
                    halukkaita[t]++;
                }
            }
        }
        int mn = Math.max(na,nt);
        neliot = new int[mn][3];
    }
    
    
    private static void viiva(PrintStream out, int pituus) {
        for (int i=0; i<pituus; i++) out.print('-');
        out.println();
    }
    

    /**
     * Tulostetaan kuka tulossa mihinkäkin aikaan 
     * @param out mihin tulostetaan
     * @param kuka mikä taulukko tulostetaan
     */
    public void tulostaKetka(PrintStream out, int[] kuka) {
        for (int t=0; t<kuka.length; t++) 
            out.printf("%3s ", kuka[t] > 0 ? ""+kuka[t]: "");
        out.println();
    }
    
    

    /**
     * Tulostetaan kuka tulossa mihinkäkin aikaan 
     * @param out mihin tulostetaan
     */
    public void tulostaKetka8(PrintStream out) {
        out.printf("%10s", "");
        tulostaKetka(out, kukaTulossa8);
    }

    /**
     * Tulostetaan kuka tulossa mihinkäkin aikaan 
     * @param out mihin tulostetaan
     */
    public void tulostaKetkaA(PrintStream out) {
        out.printf("%10s", "");
        tulostaKetka(out, kukaTulossaA);
    }

    /**
     * Tulostaa taulukon
     * @param out mihin tietovirtaan tulostetaan
     */
    public void tulosta(PrintStream out) {
        // jaettujen aikoijen asiakkaat
        out.printf("%10s", "");
        tulostaKetka(out, kukaTulossa);

        out.printf("%10s", "");
        for (int t=0; t<halukkaita.length; t++) 
            out.printf("%3d ", halukkaita[t]);
        out.println();

        out.printf("%10s", "");
        for (int t=0; t<halukkaita.length; t++) 
            out.printf("%3d ", t+1);
        out.println();

        out.printf("%10s", "");
        viiva(out, halukkaita.length*4);
        
        for (int a=0; a < ajat.length; a++) {
            out.printf("%4s ", sijoitetut[a] > 0 ? "" + sijoitetut[a] : "");
            out.printf("%3s ", a+1);
            out.printf("%1s", "|");
            var arivi = ajat[a];   
            for (int t=0; t < arivi.length; t++) {
                out.printf("%3s ", arivi[t] > 0 ? "X" : "");
            }
            out.printf("%2s", "| ");
            out.printf("%3d ", asiakkaat[a]);
            out.println();
        }
        out.printf("%10s", "");
        viiva(out, halukkaita.length*4);
    }
    
    
    /**
     * Etsitään minimiarvoa ja sen paikkaa taulukosta taulukko
     * Miniarvoon liitetään vertailutaulkon maksimi sellaisesta
     * kohdasta, jossa itse ajat-taulukossa on vielä arvo
     * @param taulukko mistä etsitään minimiä
     * @param vertailu minkä maksimi laitetaan vertailuarvoksi
     * @param suunta 1=haluttujen aikojen suuntaa, 0=asiakkaiden suuntaan
     * @return
     */
    private int[] etsiPienin(int[] taulukko, int vertailu[], int suunta) {
        int min = taulukko.length + 10;
        int max =    -1; // suunta
        int minidx = -1;
        int maxidx = -1;
        int mahdNelioita = 0; 
        
        outer:
        for (int i = 0; i < taulukko.length; i++) {
            int ni = 1;
            neliot[mahdNelioita][0] = -1;
            neliot[mahdNelioita][1] = -1;
            neliot[mahdNelioita][2] = -1;
            int arvo = taulukko[i];
            if ( arvo == 0 ) continue;
            if ( arvo <= min ) {
                if ( arvo < min ) { min = arvo; minidx = i; maxidx = -1; max = -1; } 
                for (int idx = 0; idx < vertailu.length; idx++) {
                    int vx = suunta == 1 ? ajat[idx][i] : ajat[i][idx];
                    if ( vx == 0 ) continue;
                    int vert = vertailu[idx];
                    if ( min == 2) {
                        neliot[mahdNelioita][0] = i;
                        neliot[mahdNelioita][ni++] = idx;
                        if ( ni == 3  ) mahdNelioita++;
                    }
                    if ( vert > max ) { // suunta
                        max = vert;
                        maxidx = idx;
                        minidx = i;
                        if ( min == 1 ) break outer;
                    }
                }
            }
        }
        if ( min == 2 && mahdNelioita > 0 ) {
            neliota:  // Tutkitaan jäikö 2x2 neliöitä
            for (int i=0; i<mahdNelioita; i++ ) {
                for (int j=i+1; j<mahdNelioita; j++) {
                    if ( neliot[i][1] == neliot[j][1] && neliot[i][2] == neliot[j][2] ) {
                        minidx = neliot[i][0];
                        maxidx = neliot[i][1];
                        min = 1;
                        max = 1;
                        break neliota;
                    }
                }
            }
        }
        
        int mt = suunta == 1 ? minidx : maxidx;    
        int ma = suunta == 0 ? minidx : maxidx;    
        return new int[] {min, max, ma, mt};
    }
    
    
    /**
     * Sijoitetaan paikkaan a,t 0 ja samalla vähennetään laskureita.
     * Jos joku rivi/sarake tulee 0:ksi muuhun kohtaan, palautetaan false.
     * @param a mille riville (asiakkaat)
     * @param t mihin aikaan (time)
     * @return false jos tulee ratkeamaton
     */
    private boolean sijoita(int a, int t) {
        boolean ok = true;
        for (int ai=0; ai < asiakkaat.length; ai++) {
            if ( ajat[ai][t] > 0 ) {
                asiakkaat[ai]--;
                if ( ai != a && asiakkaat[ai] <= 0 ) ok = false;
                ajat[ai][t] = 0;
            }
        }
        for (int ti=0; ti < halukkaita.length; ti++) {
            if ( ajat[a][ti] > 0 ) {
                halukkaita[ti]--;
                if ( ti != t && halukkaita[ti] <= 0 ) ok = false;
                ajat[a][ti] = 0;
            }
        }
        asiakkaat[a] = 0;
        halukkaita[t] = 0;
        sijoitetut[a] = t+1;
        kukaTulossa[t] = a+1;
        sijoitettuAsiakkaita++;
        sijoitettuAikoja++;
        return ok;
    }

    
    /**
     * @param out mihin tulostetaan sijoittetujen määrä?
     */
    public void tulostaSijoitettuja(PrintStream out) {
        String s = getSeed();
        if ( s.length() > 0 ) s = ", siemen: " + s;
        out.printf("Asiakkaita: %d, aikoja: %d%s\n",
                sijoitettuAsiakkaita, sijoitettuAikoja, s);
    }
    
    
    private String int0to1(int[] t) {
        return String.format("%d,%d: [%d, %d]", t[0], t[1], t[2]+1, t[3]+1);
    }
    
    /**
     * Otetaan seuraava askel.
     * Etsitään "kapein" sarake jolla on eniten valintoja pystysuunnassa (1).
     * Etsitään "matalin" rivi jolla on eniten valintoja vaakasuunnassa (0).
     * Näistä valitaan pienempi tai jos yhtäsuuria, valitaan se jossa on eniten toisen suunan valintoja.
     * Jos jo ensimmäinen minimi on 1, ei toista kannata etsiä ja miniminä käytetään ensimmäistä.
     * Siirretään tämä valinnaksi, eli merkitään kuka mihin aikaan ja tyhjennetään vastaava
     * rivi ja sarake.
     * Askel epäonistuu jos se tuottaa 0 johonkin rivi/sarekalaskuriin (asiakkaat, halukkaita)
     * paikkaan, joka ei ole sijoitettavassa kohdassa.
     * Askel epäonnistuu myös, jos minimin indeksiksi tulee -1.
     * @param out mihin tulostetaan
     * @return onnistuiko askel?
     */
    public boolean askel(PrintStream out) {
        var mint = etsiPienin(halukkaita,asiakkaat,1);
        var mina = mint;
        var min = mint;
        if ( mint[0] > 1 ) { // ykkösellä ei kannata etsiä lisää
            mina = etsiPienin(asiakkaat,halukkaita,0);
            if ( mina[0] < mint[0] ) min = mina;
            else if ( mina[0] == mint[0] && mina[1] > mint[1] ) min = mina;
        }
        if ( out != null ) {
            out.print(int0to1(mint) + " ");
            if ( mina != mint ) {
                out.print(int0to1(mina) + " -> ");
                out.print(int0to1(min) );
            }    
            out.println();
        }    
        int a = min[2], t = min[3];
        if (a < 0 || t < 0) {
            return false;
        }
        if ( !sijoita(a, t) ) return false;
        if ( out != null ) tulosta(out);
        return true;
    }
    
    
    /**
     * Yriteään ratkaista ongelma minimivaihtoehtoja etsien.
     * Virhetilanteessa ongelma jätetään virheen tilaan. 
     * @param tout mihin tuostetan askeleet
     * @return true jos ratkeaa, false jos ei
     */
    public boolean ratkaise(PrintStream tout) {
        for ( int i = 0; i < getNT(); i++) 
            if ( !askel(tout) ) return false;
        return true;
    }
    
    /**
     * Ensimmäisen ei tyhjän paikka jonossa
     * @param s jono josta etsitään
     * @return ensimmäisen ei-tyhjän indeksi
     */
    public static int ensimmaisenMerkinPaikka(String s) {
        for(int i = 0; i < s.length(); i++){
            if (!Character.isWhitespace(s.charAt(i))) {
                return i;
            }
        }
        return -1;
    }    
    
    
    private static int[][] checkParturiFormat(List<String> lines) throws IOException {
        int ekaViiva = -1;
        int vikaViiva = -1;
        int spaces = -1;
        for (int i = 0; i < lines.size(); i++) {
            String s = lines.get(i);
            if ( s.indexOf("---------") >= 0 ) {
                if ( ekaViiva < 0 ) ekaViiva = i; else {
                    vikaViiva = i;
                    break;
                }
                
                continue;
            }
            int tolppa = s.indexOf('|');
            if ( tolppa >= 0 ) spaces = tolppa;
        }
        if ( ekaViiva < 0 || spaces < 0 ) return null;
        
        if ( vikaViiva < 0 ) vikaViiva = lines.size();
        int na = vikaViiva - ekaViiva -1;
        if ( na < 2 ) throw new IOException("Muoto rikki riveissä!");

        String s = lines.get(ekaViiva+1);
        int loppuTolppa = s.indexOf('|', spaces+1);
        if ( loppuTolppa < 0 ) loppuTolppa = s.length();
        int nt = loppuTolppa - spaces - 1;
        if ( nt < 4 ) throw new IOException("Muoto rikki sarakkeissa!");
        nt = nt / 4;
        int ajat[][] = new int[na][nt];
        
        for (int ia=0; ia < na; ia++) {
            String st = lines.get(ia+ekaViiva+1);
            int alku = spaces + 1;
            for (int it=0; it < nt; it++) {
                String solu = st.substring(alku, alku+4).trim();
                if ( solu.length() > 0 ) ajat[ia][it] = 1;
                alku += 4;
            }
        }
        
        return ajat;
    }
    
    /**
     * Luetaan matriisi tiedostosta
     * @param name tiedoston nimi
     * @return matriisi taulukkona
     */
    public static int[][] readFromFile(String name) {
        try {
            List<String> lines = Files.readAllLines(Paths.get(name), StandardCharsets.UTF_8);
            int[][] ajat = checkParturiFormat(lines);
            if ( ajat != null ) return ajat;
            
            int ny = lines.size();
            int nx = 0;
            int spaces = 1000;
            for (String s: lines) {
                int sp = ensimmaisenMerkinPaikka(s);
                if ( sp < spaces ) spaces = sp;
            }

            for (int i = 0; i < lines.size(); i++) {
                String s = lines.get(i);
                if ( s.indexOf("\t") >= 0 ) {
                    var palat = s.split("\t");
                    var sb = new StringBuilder();
                    for (var pala: palat)
                        sb.append(pala.length() > 0 ? pala : " ");
                    s = sb.toString();
                }
                s = s.substring(spaces).replaceAll("\\s+$", "").replaceAll("[\r\n\t]+", "");
                if ( s.length() > nx ) nx = s.length();
                lines.set(i, s);
            }
            
            ajat = new int[ny][nx];
            

            for (int a = 0; a < lines.size(); a++) {
                var s = lines.get(a);
                for (int t=0; t < s.length(); t++ ) {
                    char c = s.charAt(t);
                    if ( c != ' ' && c != '0' && c != 160 ) ajat[a][t] = 1;
                }
                
            }
            
            return ajat;
        } catch (IOException e) {
            System.err.println(e.getMessage());
            System.exit(1);
            return null;  // tyhjä return jotta saa valitukset pois
        }
    }
    
    
    /**
     * Asiakasrivin tiedot 
     */
    public static class Asiakas implements Comparable<Asiakas>{
        
        private int[] ajat;      // asiaakkan aikojen määrä  
        private int n;           //  asiakkaan aikojen määrä rivillä  
        private int aika;        // asiakkaalle nyt varattu aika          
        private int nro;         // asiakkaan alkuperäinen nro jos sortataan
        private int[] alkupRivi; // asiakkaan alkupeärinen vapaiden aikojen rivi;
        private int[] monesko;   // taulukko jolla ajan indeksistä päästään asiakkaan ajan indeksiin.
        /**
         * @param ajat aikataulukko josta otetaan nt kpl
         * @param nt aikojen maxmäärä
         * @param nro alkuperäinen nro 
         * @param alkupRivi viite alkuperäisen matriisin riviin 
         */
        public Asiakas(int[] ajat, int nt, int nro, int[] alkupRivi) {
            this.ajat = new int[nt];
            this.alkupRivi = alkupRivi;
            System.arraycopy(ajat, 0, this.ajat, 0, nt);
            n = nt;
            aika = 0;
            this.nro = nro;
            this.monesko = new int[alkupRivi.length];
            int m = 0;
            for (int i = 0; i < alkupRivi.length; i++) {
                if ( alkupRivi[i] > 0 ) monesko[i] = m++;
            }
        }
        
        /**
         * @param t lisättävä aika
         */
        public void add(int t) {
            if ( n >= ajat.length ) return;
            ajat[n++] = t;
        }

        @Override
        public int compareTo(Asiakas a2) {
            return n - a2.n;
        }
        
        /**
         * @param t aika jota tutkitaan
         * @return kelpaako asiakkaalle aika t
         */
        public boolean kelpaako(int t) {
            return alkupRivi[t] > 0;
        }
        
        /**
         * @param aika joka laitetaan asiakkaalle
         */
        public void setAika(int aika) {
           int i = monesko[aika];
           this.aika = i;
        }
        
        /**
         * @return asikkalle merkattua aika
         */
        public int getAika() {
            return ajat[aika];
        }
        
        /**
         * @return jos siirto onnistuu.  Ei siirry jos ei onnistu
         */
        public boolean next() {
            if ( aika >= n-1) return false;
            aika++;
            return true;
        }
        
        
        /**
         *  siirtyy rivin alkuun
         */
        public void reset() {
            aika = 0;
        }
        
    }

    
    /**
     * Alkio permutaatiotiedon tallentamiseksi
     */
    public class PermAlkio implements Comparable<PermAlkio>{
        private int n;
        private int nro;
        /**
         * @param n      Lajitteluarvo
         * @param nro    tallennettava tieto
         */
        public PermAlkio(int n, int nro) {
            this.n = n;
            this.nro = nro;
        }
        
        @Override
        public int compareTo(PermAlkio t) {
            // TODO Auto-generated method stub
            return this.n - t.n;
        }
        
        /** @return nro   */
        public int getNro() { return nro; }
    }
    

    private void tulostaTiedot(PrintStream out) {
        String s = getSeed();
        if ( s.length() > 0 ) s = "seed = " + s + "L;   ";
        out.printf("%sn = %d; tayttoaste = %d;\n", s, ajat.length, prosentti);
    }
    
    private void tarkista(int kuka[]) {
        // int na = alkup.length;
        int nt = alkup[0].length;
        int[] ketka = new int[nt];
        System.arraycopy(kuka, 0, ketka, 0, nt);
        Arrays.sort(ketka);
        for (int t=0; t<nt; t++) 
            if ( ketka[t] != t + 1) {
                tulosta(System.out);
                tulostaTiedot(System.out);
                System.out.println("Vastauksessa: " + Arrays.toString(kuka) + " ei ole kaikkia!");
                System.exit(1);
            }
                
        
        for (int t=0; t < nt; t++) {
            int a = kuka[t]-1;
            if ( alkup[a][t] == 0 ) {
                tulosta(System.out);
                tulostaTiedot(System.out);
                System.out.println("Vastauksessa: " + Arrays.toString(kuka) + " henkilöllä " + a + "ei ole aikaa!");
                System.exit(1);
            }
        }
    }
    
    
    /**
     * Ratkaistaan 8-tornin tekniikalla
     * @param perm käyetään sarakkeiden permutaatiota
     * @return ratkeaako
     */
    public boolean ratkaise8torni(boolean perm) {
        int na = alkup.length;
        int nt = alkup[0].length;
        Asiakas[] as = new Asiakas[na];
        int asiakkaat8[] = new int[na];
        int[] kuka = new int[nt];  // kuka asiaks on tässä ajassa nyt;
        int[] aajat = new int[nt];  // työtaulukko aikojen luomiseksi     
        kukaTulossa8 = new int[nt];
        PermAlkio[] lajitellut_ajat = new PermAlkio[nt];
        
        for (int t=0; t < nt; t++) {
            int maara = 0;
            for (int a=0; a < na; a++) {
                if ( alkup[a][t] > 0 ) maara++;
            }
            if ( maara == 0 ) return false;
            lajitellut_ajat[t] = new PermAlkio(maara, t);
        }
        
        if ( perm ) Arrays.sort(lajitellut_ajat);
        
        for (int a=0; a < na; a++) {
            int ant = 0;
            var arivi = alkup[a];   
            for (int t=0; t < arivi.length; t++) {
                int pt = lajitellut_ajat[t].getNro();
                if ( arivi[pt] > 0 ) {
                    asiakkaat8[a]++;
                    aajat[ant++] = pt;
                }
            }
            if ( asiakkaat8[a] == 0 ) return false;
            as[a] = new Asiakas(aajat, ant, a, alkup[a]);
        }
        
        
        for (int t = 0; t < kuka.length; t++) kuka[t] = -1;
        
        Arrays.sort(as);

        int a = 0;
        while (a < na) {
            Asiakas aa = as[a];
            int t = aa.getAika();
            if ( kuka[t] < 0 ) { kuka[t] = a; a++; continue; }
            if ( aa.next() ) continue;
            do {
                aa.reset();  // siirto ei onnistunut, puretaan
                a--;
                if ( a < 0 ) return false;
                aa = as[a];
                int tp = aa.getAika();
                kuka[tp] = -1;
            } while (!aa.next());
       }
        
       for (int t = 0; t < kuka.length; t++) { // siirretään tulos
           int pt = t; // lajitellut_ajat[t].getNro(); 
           kukaTulossa8[t] = as[kuka[pt]].nro + 1;
       }
        
       tarkista(kukaTulossa8);
       
       return true;
    }
    
    /**
     * Ratkaistaan tekniikalla, missä k ekaa asiakasta on kohdallaan lisätään
     * seuraava.  Jos ei mene paikalleen, niin aloitetaan käyttämällä tätä ekaksi.
     * Jos askeleessa k-sama asiakas tulee uudelleen sijoitettavaksi, niin ratkaisua ei löydy.
     * @return ratkeaako
     */
    public boolean ratkaiseAlkuSijoitettu() {
        int na = alkup.length;
        int nt = alkup[0].length;
        Asiakas[] as = new Asiakas[na];
        int[] kuka = new int[nt];   // kuka asiaks on tässä ajassa nyt;
        int[] aajat = new int[nt];  // työtaulukko aikojen luomiseksi     
        kukaTulossaA = new int[nt];
        
        for (int a=0; a < na; a++) {
            int aikoja = 0;
            var arivi = alkup[a];   
            for (int t=0; t < arivi.length; t++) {
                int pt = t;
                if ( arivi[pt] > 0 ) {
                    aajat[aikoja++] = pt;
                }
            }
            if ( aikoja == 0 ) return false;
            as[a] = new Asiakas(aajat, aikoja, a, alkup[a]);
        }
        
        
        for (int t = 0; t < kuka.length; t++) kuka[t] = -1;
        
        // Arrays.sort(as);

        seuraavaAsiakas:
        for (int a=0; a<na; a++) {
            Asiakas asiakas = as[a];
            do {
                int t = asiakas.getAika();
               if ( kuka[t] < 0 ) { kuka[t] = a; continue seuraavaAsiakas; }
            } while  ( asiakas.next() );
            // seuraavaksi kiinnostaa vapaat sarakkeet, joihin joku asiaskas
            // voisi siirtyä ja asiakas on sarakkeessa joka samalla kelpaa a:lle
            for (int t=0; t<nt; t++) {
                if ( kuka[t] >= 0 ) continue;  // sarake on jo varattu
                for (int ja=a-1; ja>=0; ja--) {
                    Asiakas jokuAsiakas = as[ja];
                    if ( !jokuAsiakas.kelpaako(t) ) continue;
                    int nytAika = jokuAsiakas.getAika(); 
                    if ( asiakas.kelpaako(nytAika)) { 
                        asiakas.setAika(nytAika);
                        jokuAsiakas.setAika(t);
                        kuka[nytAika] = a;
                        kuka[t] = ja;
                        continue seuraavaAsiakas;
                    }
                    kuka[nytAika] = -1;
                    jokuAsiakas.setAika(t);
                    kuka[t] = ja;
                    t = nytAika;
                }
            }
            return false;
       }
        
       for (int t = 0; t < kuka.length; t++) { // siirretään tulos
           int pt = t; 
           kukaTulossaA[t] = as[kuka[pt]].nro + 1;
       }
        
       tarkista(kukaTulossaA);
       
       return true;
    }

    /** lyhenne todelle */
    public static final boolean T = true; 
    /** lyhenne epätodelle */
    public static final boolean F = false; 
    
    /**
     * Kokeillaan parturia
     * @param args tiedoton nimi
     */
 // BYCODEBEGIN
    public static void main(String[] args) {
      int kierroksia = 0;
      int epaonistuneita = 0;
      do {
        int n = 5;                   // ongelman koko
        int tayttoaste = 30;          // täyttöaste prosentteina
        boolean sallitaanko0 = F;     // sallitaanko arvonnassa tyhjät rivit tai sarakkeet?

        boolean geometrinen    = F;   // tutkitaanko geimetriseen hakuun perustuvalla algortimilla     
        boolean torni8         = T;   // tutkitaanko 8 tornin algoritmilla? 
        boolean alkusijoittelu = T;   // tutkitaanko alkusijoitteluun perustuvalla algortimilla? 

        boolean tulostalku = F;  // vaihda T jos haluat ko tulosteen ja F jos et
        boolean lopputulos = F;
        boolean valitulos  = F;
        
        long seed = 0; // Siemen 0 arpoo satunnaisen taulukon. 
        
        seed = 333290472414851117L;   n = 5; tayttoaste = 30;
        // seed = -2187047505996384721L;   n = 5; tayttoaste = 30;
        // seed = 8634722233411505269L;   n = 5; tayttoaste = 30;

        // seed = 7907707148936968229L;   n = 5; tayttoaste = 30;
        // seed = -3996502292159482067L;   n = 12; tayttoaste = 30;
        // seed = 1680162488338029610L;  n = 7;  tayttoaste = 30; // ratkeava, mutta ei neliön suojelullakaan
        // seed = 3517271246629570719L;   n = 7;  tayttoaste = 30; // ratkeava, mutta ei neliön suojelullakaan
        // seed = 5189488312527981069L;   n = 8;  tayttoaste = 30; // oikeasti ratkeamaton 
        // seed = 8648846363269165697L;   n = 12; tayttoaste = 30; // oikeasti ratkeamaton
        // seed = -8194518275280218929L;  n = 12; tayttoaste = 40; // ratkeava, mutta ei mene edes neliön suojelulla 
        
        var bs = new ByteArrayOutputStream();
        var sout = new PrintStream(bs);
        var out = System.out;
        if ( lopputulos)  sout = out;  
        PrintStream tout = null;
        if ( valitulos ) tout = sout;   
        
        long t1 = System.nanoTime();
        Parturi parturi;
        if ( args.length > 0 )  parturi = new Parturi(readFromFile(args[0]));
        else parturi = new Parturi(n, tayttoaste, seed, sallitaanko0);
        long t2 = System.nanoTime();
        System.out.printf("Alustus:         %5.1f\n", (t2-t1)/1.0e9);

        sout.println(parturi.getSeed());
        if (tulostalku) parturi.tulosta(sout);
        
        boolean onnistuiG = true;
        boolean onnistuiT = true;
        boolean onnistuiA = true;
        kierroksia++;
        if ( geometrinen ) {
            t1 = System.nanoTime();
            onnistuiG = parturi.ratkaise(tout); 
            t2 = System.nanoTime();
            System.out.printf("Minimointi:      %5.1f  %d/%d\n", (t2-t1)/1.0e9, epaonistuneita, kierroksia);
        }
        if ( torni8 ) {
            t1 = System.nanoTime();
            onnistuiT = parturi.ratkaise8torni(true); 
            t2 = System.nanoTime();
            System.out.printf("Torni 8:         %5.1f\n", (t2-t1)/1.0e9);
        }
        if ( alkusijoittelu ) {
            t1 = System.nanoTime();
            onnistuiA = parturi.ratkaiseAlkuSijoitettu(); 
            t2 = System.nanoTime();
            System.out.printf("Alkusijoittelu:  %5.1f\n", (t2-t1)/1.0e9);
        }
        
        
        if ( alkusijoittelu && torni8 ) {
            if ( onnistuiA != onnistuiT ) {
                parturi.tulosta(System.out);
                System.out.println("Alkusijoittelu ja torni8 ovat eri mieltä!!!");
                parturi.tulostaTiedot(System.out);
                System.exit(1);
            }
        }

        if ( onnistuiG && onnistuiT && onnistuiA) {
            parturi.tulosta(sout);
            if ( torni8 ) parturi.tulostaKetka8(out);
            if ( alkusijoittelu) parturi.tulostaKetkaA(out);
        }
        else {   
            String s = String.format("Täyttö: %d", tayttoaste);
            if ( args.length > 0 ) s = args[0];
            out.printf("Tehtävä ei ratkennut! %s n: %d\n", s, n);
            if ( onnistuiT && torni8 || onnistuiA && alkusijoittelu) {  // tarkistetaan onko toinen samaa mieltä
                out.println(bs.toString());
                parturi.tulostaSijoitettuja(out);
                parturi.tulosta(out);
    
                if ( torni8 ) parturi.tulostaKetka8(out);
                if ( alkusijoittelu) parturi.tulostaKetkaA(out);
                
                epaonistuneita++;
                // System.exit(1);
            }
        }

        if ( seed !=0 ) break;  // asetetulla siemenellä ei järkeä tehdä enempää
      } while (args.length == 0); // samoin jos käyettiin valmista tiedostoa
    }
 // BYCODEEND
}
